@@ -1,4 +1,4 @@
-# $Id: Request.pm 196 2005-07-28 03:27:36Z rcaputo $
+# $Id: Request.pm 203 2005-08-02 16:02:33Z rcaputo $
 
 package POE::Component::Client::HTTP::Request;
 use strict;
@@ -121,7 +121,8 @@ sub new {
     # This request qualifies for proxying.  Replace the host and port
     # with the proxy's host and port.  This comes after the Host:
     # header is set, so it doesn't break the request object.
-    ($host, $port) = $params{Proxy}->[rand @{$params{Proxy}}];
+    ($host, $port) = @{$params{Proxy}->[rand @{$params{Proxy}}]};
+
     $using_proxy = 1;
   }
   else {
@@ -516,6 +517,16 @@ sub connect_error {
   return;
 }
 
+sub host { shift->[REQ_HOST] }
+
+sub port { shift->[REQ_PORT] }
+
+sub scheme {
+  my $self = shift;
+
+  $self->[REQ_USING_PROXY] ? 'http' : $self->[REQ_REQUEST]->uri->scheme;
+}
+
 sub DESTROY {
   my ($self) = @_;
 
